package com.aldiferdiyan.xmpp.flutter_xmpp;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.util.Log;
import android.widget.Toast;

import java.util.HashMap;
import java.util.Map;

import io.flutter.app.FlutterActivity;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

public class FlutterXmppPlugin extends FlutterActivity implements MethodCallHandler, EventChannel.StreamHandler {

  private static final String TAG ="flutter_xmpp";
  public static final Boolean DEBUG = true;


  private static final String CHANNEL = "flutter_xmpp/method";
  private static final String CHANNEL_STREAM = "flutter_xmpp/stream";

  private static Activity activity;

  private String jid_user = "";
  private String password = "";
  private String host = "";
  private Integer port = 0;
  private BroadcastReceiver mBroadcastReceiver = null;
  private String current_stat = "STOP";
  FlutterXmppPlugin(Activity activity) {
    this.activity = activity;
  }

  public static void registerWith(Registrar registrar) {

    //method channel
    final MethodChannel method_channel = new MethodChannel(registrar.messenger(), CHANNEL);
    method_channel.setMethodCallHandler(new FlutterXmppPlugin(registrar.activity()));

    //event channel
    final EventChannel event_channel = new EventChannel(registrar.messenger(), CHANNEL_STREAM);
    event_channel .setStreamHandler(new FlutterXmppPlugin(registrar.activity()));

  }


  // ****************************************
  // stream
  @Override
  public void onListen(Object auth, EventChannel.EventSink eventSink) {
    if(DEBUG) {
      Log.w(TAG, "adding listener");
      Log.d(TAG, auth.toString());
    }
    if (FlutterXmppConnectionService.getState().equals(FlutterXmppConnection.ConnectionState.DISCONNECTED)) {
      Map<String, Object> auth_data = (Map<String, Object>) auth;
      this.jid_user = auth_data.get("user_jid").toString();
      this.password = auth_data.get("password").toString();
      this.host = auth_data.get("host").toString();
      if (auth_data.containsKey("port")) {
        this.port = Integer.parseInt(auth_data.get("port").toString());
      }
      login();
      mBroadcastReceiver = get_message(eventSink);
      IntentFilter filter = new IntentFilter(FlutterXmppConnectionService.RECEIVE_MESSAGE);
      activity.registerReceiver(mBroadcastReceiver, filter);
    }
  }

  @Override
  public void onCancel(Object o) {
    if(mBroadcastReceiver != null) {
      if(DEBUG) {
        Log.w(TAG, "cancelling listener");
      }
      if (FlutterXmppConnectionService.getState().equals(FlutterXmppConnection.ConnectionState.CONNECTED)) {
        logout();
        activity.unregisterReceiver(mBroadcastReceiver);
        mBroadcastReceiver = null;
      }
    }
  }



  private static BroadcastReceiver get_message(final EventChannel.EventSink events) {
    return new BroadcastReceiver() {
      @Override
      public void onReceive(Context context, Intent intent) {
        String action = intent.getAction();
        switch (action) {
          case FlutterXmppConnectionService.RECEIVE_MESSAGE:
            String from = intent.getStringExtra(FlutterXmppConnectionService.BUNDLE_FROM_JID);
            String body = intent.getStringExtra(FlutterXmppConnectionService.BUNDLE_MESSAGE_BODY);
            if(DEBUG) {
              Log.d(TAG, "msg : " + from + " : " + body);
            }
            Map<String, Object> build = new HashMap<>();
            build.put("from", from);
            build.put("body", body);
            events.success(build);
            break;


        }
      }
    };
  }


  // ****************************************
  // method call
  @Override
  public void onMethodCall(MethodCall call, Result result) {

    // send_message
    if(call.method.equals("send_message")) {
      if (!call.hasArgument("to_jid") || !call.hasArgument("body")) {
        result.error("MISSING", "Missing argument to_jid / body.", null);
      }
      String to_jid = call.argument("to_jid");
      String body = call.argument("body");
      send_message(body, to_jid);

      // send to self, for sync current msg all user current login from all resource
      send_message(body, this.jid_user);

      result.success("SUCCESS");

    }else if(call.method.equals("current_state")){
      String state = "UNKNOWN";
      switch (FlutterXmppConnectionService.getState()){
        case CONNECTED:
          state = "CONNECTED";
          break;
        case AUTHENTICATED:
          state = "AUTHENTICATED";
          break;
        case CONNECTING:
          state = "CONNECTING";
          break;
        case DISCONNECTING:
          state = "DISCONNECTING";
          break;
        case DISCONNECTED:
          state = "DISCONNECTED";
          break;
      }

      if(DEBUG){
        Log.d(TAG,state);
      }
      result.success(state);

    }else {
      result.notImplemented();
    }
  }

  // login
  private void login(){
    if (FlutterXmppConnectionService.getState().equals(FlutterXmppConnection.ConnectionState.DISCONNECTED)) {
      Intent i = new Intent(activity, FlutterXmppConnectionService.class);
      i.putExtra("jid_user", jid_user);
      i.putExtra("password", password);
      i.putExtra("host", host);
      i.putExtra("port", port);
      activity.startService(i);
    }
  }

  private void logout(){
    if (FlutterXmppConnectionService.getState().equals(FlutterXmppConnection.ConnectionState.CONNECTED)) {
      Intent i1 = new Intent(activity, FlutterXmppConnectionService.class);
      activity.stopService(i1);
    }
  }

  // send message to JID
  private void send_message(String msg,String jid_user){
      Log.d(TAG,"Current Status : " + FlutterXmppConnectionService.getState().toString());
    if (FlutterXmppConnectionService.getState().equals(FlutterXmppConnection.ConnectionState.CONNECTED)) {
      if(DEBUG) {
        Log.d(TAG, "ngirim pesan ke : " + jid_user);
      }
      Intent intent = new Intent(FlutterXmppConnectionService.SEND_MESSAGE);
      intent.putExtra(FlutterXmppConnectionService.BUNDLE_MESSAGE_BODY, msg);
      intent.putExtra(FlutterXmppConnectionService.BUNDLE_TO, jid_user);
      activity.sendBroadcast(intent);
    } else {
      if(DEBUG) {
        Log.d(TAG, "Tidak terhubung ke server");
      }
    }
  }


  private void set_presence(){

  }

  private void get_presence(){

  }


}
